using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;

namespace ThumbViewerControl
{
	public class ThumbViewer : UserControl
	{		
		ArrayList images = new ArrayList(); // TlCۑp

		const int baseSize = 150; // ێrbg}bṽTCY
		const int padding = 2; // Obhł̃TlC̗]

		int thumbSize; // \̃TlC̃TCYi`j
		int cols; // Obh̗
		int rows; // Obh̍s
		int gridSize; // Obh̃TCY
		int selectedIndex; // ݑIĂTlC
		
		Pen framePen; // g\p̃y
		Pen frameErasePen; // gp̃y

		/// <summary>
		/// KvȃfUCiϐłB
		/// </summary>
		private System.ComponentModel.Container components = null;

		public ThumbViewer()
		{
			// ̌ĂяóAWindows.Forms tH[ fUCiŕKvłB
			InitializeComponent();

			// TODO: InitComponent ĂяǒɏǉĂB

			// _uobt@Oɂ`̍
			this.SetStyle(ControlStyles.DoubleBuffer, true);
			this.SetStyle(ControlStyles.UserPaint, true);
			this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);

			this.BackColor = SystemColors.Window;

			framePen = new Pen(SystemColors.Highlight, 2);

			// ftHg̃TlC̃TCYݒ
			thumbSize = 64;

			// IĂTlCȂ
			selectedIndex = -1;
			updateRowsCols();
		}

		/// <summary>
		/// gpĂ郊\[XɌ㏈s܂B
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if( components != null )
					components.Dispose();
			}
			base.Dispose( disposing );
		}

		#region R|[lg fUCiŐꂽR[h 
		/// <summary>
		/// fUCi T|[gɕKvȃ\bhłB̃\bh̓e 
		/// R[hnGfB^ŕύXȂłB
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// ThumbViewer
			// 
			this.Name = "ThumbViewer";

		}
		#endregion


		public void Add(Image img) 
		{
			Bitmap bmp;
			
			// TlC쐬ăRNVŕێ
			if (img.Width > img.Height) 
			{
				bmp = new Bitmap(img, baseSize, baseSize  * img.Height / img.Width);
			} 
			else 
			{
				bmp = new Bitmap(img, baseSize  * img.Width / img.Height, baseSize);
			}
			images.Add(bmp);
			updateRowsCols();

			// ǉꂽTlC\
			using (Graphics g = this.CreateGraphics()) 
			{
				drawOne(g, images.Count - 1, false);
			}
		}

		// w肳ꂽ̈̒ɉ摜`
		void drawImageCenter(Graphics g, Image image, int x, int y, int w, int h) 
		{
			float fw = (float)w / (float)image.Width;
			float fh = (float)h / (float)image.Height;

			float scale = Math.Min(fw, fh);
			fw = image.Width * scale;
			fh = image.Height * scale;

			// g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.High; // 荂掿
			g.DrawImage(image, 
				x + (w - fw) / 2, 
				y + (h - fh) / 2,
				fw, fh);
		}

		// w肳ꂽ̈ɋ``^
		void drawFrameRectangle(Graphics g, int x, int y, int w, int h, bool draw)
		{
			frameErasePen = new Pen(this.BackColor, 2);
			Pen p = draw ? framePen : frameErasePen;
			g.DrawRectangle(p, x + 2, y + 2, w - 4, h - 4);
			frameErasePen.Dispose();
		}

		// w肳ꂽCfbNXԍ̃TlC\
		void drawOne(Graphics g, int index, bool selected)
		{
			int col = index % cols;
			int row = index / cols;

			drawImageCenter(
				g, (Image)images[index], 
				col * gridSize + (gridSize - thumbSize) / 2 + padding, 
				row * gridSize + (gridSize - thumbSize) / 2 + padding + this.AutoScrollPosition.Y,
				thumbSize - padding * 2, thumbSize - padding * 2);

			if (selected) 
			{
				// g\
				drawFrame(g, index, true);
			}
		}

		// w肳ꂽCfbNXԍ̘g\^
		void drawFrame(Graphics g, int index, bool draw)
		{
			int col = index % cols;
			int row = index / cols;

			drawFrameRectangle(g, 
				col * gridSize,
				row * gridSize + this.AutoScrollPosition.Y, 
				gridSize, gridSize, draw);
		}

		// Obh̗񐔂ƍsvZ
		void updateRowsCols() 
		{
			if (this.ClientSize.Width <= 0)  // ŏȂ
			{
				return;
			}

			cols = this.ClientSize.Width / (thumbSize + padding * 2);
			if (cols == 0)
				cols = 1;

			gridSize = this.ClientSize.Width / cols;

			rows = images.Count / cols;
			if (images.Count % cols > 0)
				rows++;

			if (thumbSize > gridSize - padding * 2) 
			{
				thumbSize = gridSize - padding * 2;
			}

			// TlC̕\̈ݒ
			this.AutoScrollMinSize = new Size(0, rows * gridSize);
		}

		// ݑIĂCfbNXԍ̎擾iIF-1j
		public int SelectedIndex 
		{
			get { return selectedIndex; }
		}

		// TlC`悷TCY̐ݒƎ擾
		public int ThumbnailSize 
		{
			get { return thumbSize; }
			set 
			{
				thumbSize = value;

				if (images.Count == 0)
					return;

				updateRowsCols();
				Invalidate();
			}
		}

		// TlC摜Ɖʂ̃NA
		public void Clear()
		{
			foreach (Image img in images) 
			{ 
				img.Dispose(); 
			}
			images.Clear();
			selectedIndex = -1;

			updateRowsCols();
			Invalidate(); 
		}

		protected override void OnPaint(PaintEventArgs e)
		{			
			base.OnPaint (e);

			int rowStart = (-this.AutoScrollPosition.Y + e.ClipRectangle.Y) / gridSize;
			int rowEnd = (-this.AutoScrollPosition.Y + e.ClipRectangle.Y + e.ClipRectangle.Height) / gridSize + 1;

			for (int i = rowStart * cols; i < Math.Min(rowEnd * cols, images.Count); i++) 
			{
				drawOne(e.Graphics, i, i == selectedIndex);
			}
		}

		protected override void OnResize(EventArgs e)
		{
			base.OnResize(e);

			updateRowsCols();
			Invalidate();
		}

		protected override void OnMouseDown(MouseEventArgs e)
		{
			int col = e.X / gridSize;
			int row = (e.Y - this.AutoScrollPosition.Y) / gridSize;

			int newSelected = row * cols + col;

			// TlC݂̑ȂꏊNbNꂽꍇ
			if (newSelected > images.Count - 1) 
			{
				return;
			}

			// IĂTlCύX
			if (newSelected != selectedIndex) 
			{
				using (Graphics g = this.CreateGraphics()) 
				{
					if (selectedIndex != -1) 
					{						
						drawFrame(g, selectedIndex, false); // g
						drawOne(g, selectedIndex, false); // gȂŕ`
					}
					selectedIndex = newSelected;
					drawOne(g, selectedIndex, true); // gtŕ`
				}
			}

			// CfbNXύXɃCxgEnhĂяo悤ɂ
			base.OnMouseDown(e);			
		}
	}
}
